package	jp.co.sds_corp.report;

import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;

import javax.servlet.http.HttpServletResponse;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.WarekiFormat;
import jp.co.sds_corp.util.script.ScriptParserFactory;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDataFormat;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFPrintSetup;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.CellRangeAddress;

import com.lowagie.text.Cell;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.PageSize;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfWriter;

/**
<a href="http://jakarta.apache.org/poi/">Jakarta POI</a>
jexcelapiのcom.lowagie.text
を使用して、Excel帳票を出力します。
@@@ colspan,rowspanは指定できるが、rowの高さが指定できない。
<pre>
&lt;Report
	name="ReportTest"
	type="Excel"
	template="${Report.dir}/table.xls"
	sheet="Sheet1"&gt;
	&lt;Script ... /&gt;
&lt;/Report&gt;
</pre>
Script要素については、{@link ScriptParserFactory}を参照してください。
*/
public class ExcelPDFReport1 extends ExcelReport
{
	static long dayDiff;
	static {
		try {
			CmnDate dtFr = new CmnDate( "19000101" );
			CmnDate dtTo = new CmnDate( "19700101" );
			dayDiff = dtTo.diffDays( dtFr ) + 2;
		} catch ( Exception ex ) {
		}
	}

	HashMap<Integer,Font>	mapFont = new HashMap<Integer,Font>();

	/**
	* 帳票を出力します。
	* @param	out		出力先
	*/
	public void print( OutputStream out ) throws Exception {
		if ( !bPrint ) {
			print();
		}
		HSSFPrintSetup prset = result.getSheetAt( 0 ).getPrintSetup();
		Rectangle size = PageSize.A4;
		if ( prset.getLandscape()) {
			size = size.rotate();
		}
		Document doc = new Document( size );
		PdfWriter.getInstance( doc, out );
		doc.open();
		for ( int i = 0; i < result.getNumberOfSheets(); i++ ) {
			HSSFSheet sheet = result.getSheetAt( i );
System.out.println( "sheet=" + result.getSheetName( i ));
			printSheet( doc, result, sheet );
		}
		doc.close();
	}

	void printSheet( Document doc, HSSFWorkbook book, HSSFSheet sheet )
		throws Exception
	{
		HSSFPrintSetup prset = sheet.getPrintSetup();
		Rectangle size = PageSize.A4;
		if ( prset.getLandscape()) {
			size = size.rotate();
		}
		boolean bRet = doc.setPageSize( size );
		bRet = doc.newPage();
		BaseFont bf = BaseFont.createFont(
								"HeiseiKakuGo-W5",
								"UniJIS-UCS2-H",
								BaseFont.NOT_EMBEDDED);
		int nMaxCol = 0;
		for ( int i = 0; i <= sheet.getLastRowNum(); i++ ) {
			HSSFRow row = sheet.getRow( i );
			if ( row == null ) {
				continue;
			}
			if ( nMaxCol < row.getLastCellNum() ) {
				nMaxCol = row.getLastCellNum();
			}
		}
		byte[][] pos = new byte[sheet.getLastRowNum()+1][nMaxCol];
		for ( int i = 0; i < sheet.getNumMergedRegions(); i++ ) {
			CellRangeAddress reg = sheet.getMergedRegion( i );
			for ( int j = reg.getFirstRow(); j <= reg.getLastRow(); j++ ) {
				for ( int k = reg.getFirstColumn(); k <= reg.getLastColumn(); k++ ) {
					pos[j][k] = -1;
				}
			}
			pos[reg.getFirstRow()][reg.getFirstColumn()] = 1;
		}

		HSSFDataFormat format = book.createDataFormat();

		Table table = createTable( sheet, nMaxCol );
		for ( int i = 0; i <= sheet.getLastRowNum(); i++ ) {
			HSSFRow row = sheet.getRow( i );
			if ( row == null ) {
				for ( int j = 0; j < nMaxCol; j++ ) {
					table.addCell( new Cell() );
				}
				if ( isRowBreak( sheet, i )) {
System.out.println( "break " + i );
					doc.add( table );
					doc.newPage();
					table = createTable( sheet, nMaxCol );
				}
				continue;
			}
			for ( int j = 0; j < nMaxCol; j++ ) {
				if ( pos[i][j] < 0 ) {
					continue;
				}
				HSSFCell hcell = row.getCell( j );
				if ( hcell == null ) {
					Cell cell = new Cell();
					cell.disableBorderSide( Cell.LEFT );
					cell.disableBorderSide( Cell.RIGHT );
					cell.disableBorderSide( Cell.TOP );
					cell.disableBorderSide( Cell.BOTTOM );
					table.addCell( cell );
					continue;
				}
				HSSFFont hfont = book.getFontAt( hcell.getCellStyle().getFontIndex());
				int fsize = hfont.getFontHeight() / 30;
//				int fsize = hfont.getFontHeightInPoints();
				Font font = (Font )mapFont.get( Integer.valueOf( fsize ));
				if ( font == null ) {
System.out.println( "create font " + fsize + "," + hfont.getFontHeightInPoints() );
					font = new Font( bf, fsize );
					mapFont.put( Integer.valueOf( fsize ), font );
				}
				String str = getString( hcell, format );
System.out.println( "(" + i + "," + j + ") [" + str + "]" );
				Cell cell = new Cell( new Phrase( str, font ));
				setupCell( cell, hcell );
				if ( pos[i][j] > 0 ) {
					CellRangeAddress reg = findRegion( sheet, i, j );
					if ( reg.getFirstColumn() != reg.getLastColumn()) {
						cell.setColspan( reg.getLastColumn() - reg.getFirstColumn() + 1 );
					}
					if ( reg.getFirstRow() != reg.getLastRow()) {
						cell.setRowspan( reg.getLastRow() - reg.getFirstRow() + 1 );
					}
				}
				table.addCell( cell );
			}
			if ( isRowBreak( sheet, i )) {
System.out.println( "break " + i );
				doc.add( table );
				doc.newPage();
				table = createTable( sheet, nMaxCol );
			}
		}
		table.setPadding(4);
		table.setOffset(0);
		doc.add(table);
	}

	private Table createTable( HSSFSheet sheet, int col ) throws Exception {
		Table table = new Table( col );
		table.setDefaultHorizontalAlignment(Element.ALIGN_CENTER);
		table.setDefaultVerticalAlignment(Element.ALIGN_MIDDLE);
		table.setPadding(2);

		int width[] = new int[col];
		for ( int i = 0; i < col; i++ ) {
			width[i] = (int )sheet.getColumnWidth( i );
System.out.println( "width(" + i + ")=" + sheet.getColumnWidth( i ) );
		}
		table.setWidths(width); //各カラムの大きさを設定（パーセント）
		table.setWidth(100);
		table.disableBorderSide( Cell.LEFT );
		table.disableBorderSide( Cell.RIGHT );
		table.disableBorderSide( Cell.TOP );
		table.disableBorderSide( Cell.BOTTOM );
		return table;
	}

	private CellRangeAddress findRegion( HSSFSheet sheet, int row, int col ) {
		for ( int i = 0; i < sheet.getNumMergedRegions(); i++ ) {
			CellRangeAddress reg = sheet.getMergedRegion( i );
			if ( row == reg.getFirstRow() && col == reg.getFirstColumn()) {
				return reg;
			}
		}
		return null;
	}

	private boolean isRowBreak( HSSFSheet sheet, int row ) {
		int[] rb = sheet.getRowBreaks();
		if ( rb == null ) {
			return false;
		}
		for ( int i = 0; i < rb.length; i++ ) {
			if ( row == rb[i] ) {
				return true;
			}
		}
		return false;
	}

	private String getString( HSSFCell hcell, HSSFDataFormat format ) {
		DecimalFormat df1 = null;
		WarekiFormat df2 = null;
		String str = null;
		switch ( hcell.getCellType()) {
		case HSSFCell.CELL_TYPE_STRING:
			str = hcell.getRichStringCellValue().getString();
			break;
		case HSSFCell.CELL_TYPE_NUMERIC:
			short idx = hcell.getCellStyle().getDataFormat();
			double val = hcell.getNumericCellValue();
System.out.println( "val=" + val + " format(" + idx + ")=" + format.getFormat( idx ));
			switch ( hcell.getCellStyle().getDataFormat()) {
			case 0x05:
				df1 = new DecimalFormat( "\\#,##0" );
			case 0x0:
			case 0xa4:
				if ( df1 == null ) {
					df1 = new DecimalFormat( "#" );
				}
				str = df1.format( val );
				break;
			case 0xb0:
				if ( df2 == null ) {
					df2 = new WarekiFormat( "gggY年M月d日" );
				}
			case 0xa5:
				if ( df2 == null ) {
					df2 = new WarekiFormat( "yyyy/M/d" );
				}
			case 0x0e:
				if ( df2 == null ) {
					df2 = new WarekiFormat( "M/d/yy" );
				}
				Date date = new Date( (long )(val - dayDiff) * 60 * 60 * 24 * 1000 );
				str = df2.format( date );
				break;
			default:
				str = String.valueOf( hcell.getNumericCellValue() );
			}
			break;
		case HSSFCell.CELL_TYPE_FORMULA:
			str = hcell.getCellFormula();
			break;
		}
		return str;
	}

	private void setupCell( Cell cell, HSSFCell hcell ) {
		HSSFCellStyle hstyle = hcell.getCellStyle();
// System.out.println( "color=" + hstyle.getFillBackgroundColor()
//					+ "," + hstyle.getFillForegroundColor() );
		if ( hstyle.getBorderLeft() == HSSFCellStyle.BORDER_NONE ) {
//			cell.disableBorderSide( Cell.LEFT );
		} else {
			cell.enableBorderSide( Cell.LEFT );
		}
		if ( hstyle.getBorderRight() == HSSFCellStyle.BORDER_NONE ) {
			cell.disableBorderSide( Cell.RIGHT );
		} else {
			cell.enableBorderSide( Cell.RIGHT );
		}
		if ( hstyle.getBorderTop() == HSSFCellStyle.BORDER_NONE ) {
//			cell.disableBorderSide( Cell.TOP );
		} else {
			cell.enableBorderSide( Cell.TOP );
		}
		if ( hstyle.getBorderBottom() == HSSFCellStyle.BORDER_NONE ) {
			cell.disableBorderSide( Cell.BOTTOM );
		} else {
			cell.enableBorderSide( Cell.BOTTOM );
		}

		int hal = Cell.ALIGN_LEFT;
		switch ( hstyle.getAlignment() ) {
		case HSSFCellStyle.ALIGN_RIGHT: hal = Cell.ALIGN_RIGHT; break;
		case HSSFCellStyle.ALIGN_CENTER: hal = Cell.ALIGN_CENTER; break;
		case HSSFCellStyle.ALIGN_JUSTIFY: hal = Cell.ALIGN_JUSTIFIED; break;
		case HSSFCellStyle.ALIGN_CENTER_SELECTION: hal = Cell.ALIGN_JUSTIFIED_ALL; break;
		case HSSFCellStyle.ALIGN_LEFT:
		default:
		}
		cell.setHorizontalAlignment( hal );

		int val = Cell.ALIGN_CENTER;
		switch ( hstyle.getVerticalAlignment() ) {
		case HSSFCellStyle.VERTICAL_TOP: val = Cell.ALIGN_TOP; break;
		case HSSFCellStyle.VERTICAL_BOTTOM: val = Cell.ALIGN_BOTTOM; break;
		case HSSFCellStyle.VERTICAL_JUSTIFY: val = Cell.ALIGN_JUSTIFIED; break;
		case HSSFCellStyle.VERTICAL_CENTER:
		default:
		}
		cell.setVerticalAlignment( val );
//		cell.setNoWrap( true );
	}

	/**
	* サーブレットのヘッダを設定します。
	* @param	res		HttpServletResponse
	*/
	public void setupResponse( Object res )
		throws Exception
	{
		HttpServletResponse response = (HttpServletResponse )res;
		response.setContentType( "application/pdf" );
	}

	/**
	* ファイルの拡張子を返します。
	* @return   ファイルの拡張子
	*/
	public String getExtension() {
		return "pdf";
	}

}
