package	jp.co.sds_corp.report;

import java.io.OutputStream;

import jp.co.sds_corp.report.source.ISource;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.script.IScriptParser;

public interface IReport
{
	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception;
	/**
	* 処理を終了します。
	*/
	public void exit() throws Exception;

	/**
	* 帳票の種別を返します。
	* @return	帳票の種別
	*/
	public String getType();
	/**
	* 初期情報の変換に使用するリソースを設定します。
	* @param    prop    リソース
	*/
	public void setProperty( CmnProperty prop );
	/**
	* 初期情報の変換に使用するリソースを返します。
	* @return   リソース
	*/
	public CmnProperty getProperty();
	/**
	* スクリプトを実行するオブジェクトを設定します。
	* @param	var		設定するデータ
	*/
	public void setScriptParser( IScriptParser var );
	/**
	* スクリプトを実行するオブジェクトを取得します。
	* @return	スクリプトを実行するオブジェクト
	*/
	public IScriptParser getScriptParser();
	/**
	* データを取得するオブジェクトを設定します。
	* @param	var		設定するデータ
	*/
	public void setSource( ISource var );
	/**
	* データを取得するオブジェクトを取得します。
	* @return	データを取得するオブジェクト
	*/
	public ISource getSource();
	/**
	* デバッグフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var );
	/**
	* デバッグフラグを取得します。
	* @return	デバッグフラグ
	*/
	public boolean isDebug();

	/**
	* スクリプトに変数を設定します。
	* @param    name    変数名
	* @param    obj     オブジェクト
	*/
	public void setAttribute( String name, Object obj ) throws Exception;
	/**
	* スクリプトに設定されている変数を取得します。
	* @param    name    変数名
	* @return   オブジェクト
	*/
	public Object getAttribute( String name ) throws Exception;
	/**
	* 帳票の出力結果をオブジェクトとして取得します。
	* @return	出力結果
	*/
	public Object getResult();
	/**
	* 帳票を出力します。
	*/
	public void print() throws Exception;
	/**
	* 帳票を出力します。
	* @param	out		出力先
	*/
	public void print( OutputStream out ) throws Exception;

	/**
	* サーブレットのContentTypeヘッダを取得します。
	* @return	サーブレットのContentTypeヘッダ
	*/
	public String getContentType();
	/**
	* サーブレットのヘッダを設定します。
	* @param	response	HttpServletResponse
	*/
	public void setupResponse( Object response ) throws Exception;
	/**
	* サーブレットのヘッダを設定します。
	* @param	response	HttpServletResponse
	* @param	filename	出力するファイル名
	*/
	public void setupResponse( Object response, String filename )
		throws Exception;
	/**
	* サーブレットのヘッダを設定します。
	* @param	response	HttpServletResponse
	* @param	filename	出力するファイル名
	* @param	encode		ファイル名のEncode
	*/
	public void setupResponse( Object response, String filename, String encode )
		throws Exception;
	/**
	* ファイルの拡張子を返します。
	* @return	ファイルの拡張子
	*/
	public String getExtension();

	/**
	* ファイルを読み込みます。
	* @param	file	ファイル名
	*/
	public void loadFile( String file ) throws Exception;
	/**
	* ページを読み込みます。
	* @param	sheetName	ページ名
	*/
	public void loadPage( String sheetName ) throws Exception;
	/**
	* ページに名前をつけて読み込みます。
	* @param	newName	新しいページ名
	* @param	orgName	読み込むページ名
	*/
	public void loadPage( String newName, String orgName ) throws Exception;

// ここから下は、interfaceに含める必要があるか？
// JavaScriptだけが知っていればいい話。
	/**
	* 現在処理中のページを、指定されたページに追加します。
	* @param	dstName	追加するページ
	*/
	public void appendTo( String dstName ) throws Exception;
	/**
	* 現在処理中のページを、指定されたページに追加します。
	* @param	dstName	追加するページ
	* @param	bBreak	改ページフラグ
	*/
	public void appendTo( String dstName, boolean bBreak ) throws Exception;
	/**
	* 現在処理中のページに改ページを追加します。
	*/
	public void endPage() throws Exception;
	/**
	* 現在処理中のページに改ページを追加します。
	* @param	row		改ページを追加する行
	*/
	public void endPage( int row ) throws Exception;
	/**
	* セルに値を設定します。
	* @param	pos		セルの位置
	* @param	data	設定する値
	*/
	public void setCell( String pos, Object data ) throws Exception;
	/**
	* セルに値を設定します。
	* @param	col		セルの桁
	* @param	row		セルの行
	* @param	data	設定する値
	*/
	public void setCell( int col, int row, Object data ) throws Exception;
	/**
	* セルに式を設定します。
	* @param	pos		セルの位置
	* @param	formula	設定する値
	*/
	public void setCellFormula( String pos, String formula ) throws Exception;
	/**
	* セルに式を設定します。
	* @param	col		セルの桁
	* @param	row		セルの行
	* @param	formula	設定する値
	*/
	public void setCellFormula( int col, int row, String formula) throws Exception;
	/**
	* セルに罫線を設定します。
	* @param	pos		セルの位置
	* @param	location	罫線の位置
	* @param	border	罫線
	*/
	public void setBorder( String pos, int location, Border border )
		throws Exception;
	/**
	* セルに罫線を設定します。
	* @param	col		セルの桁
	* @param	row		セルの行
	* @param	location	罫線の位置
	* @param	border	罫線
	*/
	public void setBorder( int col, int row, int location, Border border )
		throws Exception;

	/**
	* セルの値をコピーします。
	* @param	fpos	コピー元のセルの位置
	* @param	tpos	コピー先のセルの位置
	*/
	public void copyCell( String fpos, String tpos ) throws Exception;
	/**
	* セルの値をコピーします。
	* @param	fcol	コピー元のセルの桁
	* @param	frow	コピー元のセルの行
	* @param	tcol	コピー先のセルの桁
	* @param	trow	コピー先のセルの行
	*/
	public void copyCell( int fcol, int frow, int tcol, int trow )
		throws Exception;

	/**
	* セルをクリアします。
	* @param	pos		セルの位置
	*/
	public void clearCell( String pos ) throws Exception;
	/**
	* セルをクリアします。
	* @param	col		セルの桁
	* @param	row		セルの行
	*/
	public void clearCell( int col, int row ) throws Exception;

	/**
	* 現在のページに行を追加します。
	* @param	row		追加する位置
	*/
	public void addRow( int row ) throws Exception;
	/**
	* 現在のページに、templateから書式をコピーして行を追加します。
	* @param	dst		追加する位置
	* @param	src		書式の行の位置
	*/
	public void addRow( int dst, int src ) throws Exception;
	/**
	* 行の処理を終了します。
	*/
	public void endRow() throws Exception;

	/**
	* ログを出力します。（スクリプト内からの使用を想定）
	* @param	message	ログメッセージ
	*/
	public void log( String message );

}
